from .lib import download_webpage as dl
from .lib import parse_webpage as parser
from . import create_db

def process_url(url):
    create_db.create_db()    
    url_id = dl.download_url(url, 0)
    parser.parse_url(url_id, 0)


import os
from .lib import paths_management as paths
from .lib import db

def create_db():
    app_paths = paths.App_Paths()
    db_path = os.path.join(app_paths.base_script_path, 'web_downloader.db')

    if os.path.isfile(db_path) == False:
        curr_db = db.Database(db_path)
        curr_db.create_db()


from urllib import request
from zipfile import ZipFile
import os

def get_the_version_of_latest_firefox():
   response = request.urlopen('https://github.com/mozilla/geckodriver/releases/latest')
   redirected_url = response.geturl()
   version = redirected_url.split('/')[-1]
   return version

def main():
   # Create base firefox driver folder
   base_firefox_driver = 'firefox_driver'
   if not os.path.isdir(base_firefox_driver):
      os.mkdir(base_firefox_driver)

   latest_version = get_the_version_of_latest_firefox()
   geckodriver_file_name = f'geckodriver-{latest_version}-win64.zip'
   firefox_driver_url = f'https://github.com/mozilla/geckodriver/releases/download/{latest_version}/{geckodriver_file_name}'

   print('Downloading the latest Firefox driver from:')
   print(firefox_driver_url)

   request.urlretrieve(firefox_driver_url, geckodriver_file_name)
   with ZipFile(geckodriver_file_name, 'r') as zipObj:
      zipObj.extractall(f'firefox_driver\\{latest_version}')


import os
# import db
from . import db
from . import paths_management as paths
from selenium import webdriver

def get_domain(url):
    return url.replace('http://', '').replace('https://', '').replace('file://', '').split('/')[0]

def is_url_external(subject_url, url_for_checking):
    return get_domain(subject_url) != get_domain(url_for_checking)

def parse_url(url_id, depth_level):
    app_paths = paths.App_Paths()
    curr_db = db.Database()
    url_obj = curr_db.get_url(url_id)
    local_url = url_obj['file_path']

    assert app_paths.firefox_driver_path is not None

    with webdriver.Firefox(executable_path=app_paths.firefox_driver_path) as driver:
        driver.get(f'file://{local_url}')
        elems = driver.find_elements_by_tag_name('a')

        parsed_url_saved_counter = 0

        for elem in elems:
            href = elem.get_attribute('href')
            if href is not None:
                is_url_ext = 1 if is_url_external(url_obj['url'], href) else 0
                curr_db.insert_url_from_parsing(url_id, href, is_url_ext, elem.get_attribute('innerText'), depth_level)
                parsed_url_saved_counter += 1


    print(f'Number of parsed urls: {parsed_url_saved_counter}')


import os
import sqlite3
from datetime import datetime
from . import paths_management as paths

class Database:
    def __init__(self, db_path=None):
        if db_path is None:
            app_paths = paths.App_Paths()
            db_path = os.path.join(app_paths.base_script_path, 'web_downloader.db')
        self.db_path = db_path

    def check_if_db_exists(self):
        return os.path.exists(os.path.join(os.path.dirname(os.path.abspath(__file__)), self.db_path))

    def create_db(self):
        if not self.check_if_db_exists():
            with sqlite3.connect(self.db_path) as conn:
                c = conn.cursor()
                c.execute('CREATE TABLE urls (url_id integer primary key autoincrement, url text, title text, depth_level integer, file_path text, ts text)')
                c.execute('CREATE TABLE urls_from_parsing (url_from_parsing_id integer primary key autoincrement, url_id integer, url text, external_to_domain integer, title text, depth_level integer, ts text)')
                conn.commit()

    # return url_id
    def insert_url(self, url, title, depth_level, file_path):
        if self.check_if_db_exists():
            with sqlite3.connect(self.db_path) as conn:
                c = conn.cursor()
                c.execute("INSERT INTO urls (url, title, depth_level, file_path, ts) VALUES (?, ?, ?, ?, ?)", (url, title, depth_level, file_path, str(datetime.now())) )
                conn.commit()
                cursor = c.execute("SELECT last_insert_rowid() FROM urls")
                (url_id,) = cursor.fetchone()
                cursor.close()
            return url_id
        else:
            return -1

    def get_url(self, url_id):
        if self.check_if_db_exists():
            with sqlite3.connect(self.db_path) as conn:
                conn.row_factory = sqlite3.Row
                c = conn.cursor()
                cursor = c.execute( "SELECT * FROM urls WHERE url_id = ?", (url_id,) )
                url_row = cursor.fetchone()
                cursor.close()
            return url_row
        else:
            return -1

    # To-do: return affected rows
    def update_url(self, url_id, url, title, depth_level, file_path):
        if self.check_if_db_exists():
            with sqlite3.connect(self.db_path) as conn:
                c = conn.cursor()
                c.execute("UPDATE urls SET url=?, title=?, depth_level=?, file_path=? WHERE url_id=?", (url, title, depth_level, file_path, url_id) )
                conn.commit()
    
    # return urls_from_parsing_id
    def insert_url_from_parsing(self, url_id, url, external_to_domain, title, depth_level):
        if self.check_if_db_exists():
            with sqlite3.connect(self.db_path) as conn:
                c = conn.cursor()
                c.execute("INSERT INTO urls_from_parsing (url_id, url, external_to_domain, title, depth_level, ts) VALUES (?, ?, ?, ?, ?, ?)", (url_id, url, external_to_domain, title, depth_level, str(datetime.now())) )
                conn.commit()
                cursor = c.execute("SELECT last_insert_rowid() FROM urls_from_parsing")
                (urls_from_parsing_id,) = cursor.fetchone()
                cursor.close()
            return urls_from_parsing_id
        else:
            return -1

    def get_url_from_parsing(self, url_from_parsing_id):
        if self.check_if_db_exists():
            with sqlite3.connect(self.db_path) as conn:
                conn.row_factory = sqlite3.Row
                c = conn.cursor()
                cursor = c.execute( "SELECT * FROM urls_from_parsing WHERE url_from_parsing_id = ?", (url_from_parsing_id,) )
                url_tuple = cursor.fetchone()
                cursor.close()
            return url_tuple
        else:
            return -1

    # To-do: return affected rows
    def update_url_from_parsing(self, url_from_parsing_id, url_id, url, external_to_domain, title, depth_level):
        if self.check_if_db_exists():
            with sqlite3.connect(self.db_path) as conn:
                c = conn.cursor()
                c.execute("UPDATE urls_from_parsing SET url_id=?, url=?, external_to_domain=?, title=?, depth_level=? WHERE url_from_parsing_id=?", (url_id, url, title, external_to_domain, depth_level, url_from_parsing_id) )
                conn.commit()